@@ -36,19 +36,27 @@
 (defvar $use-tag-inheritance)
 (defvar $ignore-regions-regexps)
 
-(defun org-mem-parser--make-todo-regexp (keywords-string)
-  "Build a regexp from KEYWORDS-STRING.
-KEYWORDS-STRING is expected to be the sort of thing you see after
+(defun org-mem-parser--make-todo-regexp (s)
+  "Build a regexp from keyword string S.
+S is expected to be the sort of thing you see after
 a #+todo: or #+seq_todo: or #+typ_todo: setting in an Org file.
 
 The resulting regexp should be able to match any of
 the custom TODO words thus defined."
-  (thread-last keywords-string
-               (replace-regexp-in-string "(.*?)" "")
-               (string-replace "|" "")
-               (string-trim)
-               (split-string)
-               (regexp-opt)))
+  (let* ((words)
+         (pos (string-search "[ ]" s)))
+    (when pos
+      ;; Special-case Doom's [ ] keyword.
+      ;; https://github.com/orgs/doomemacs/discussions/74
+      (setq s (concat (substring s 0 pos)
+                      (substring s (+ 3 pos)))))
+    (setq words (thread-last s
+                             (replace-regexp-in-string "(.*?)" "")
+                             (string-replace "|" "")
+                             (string-trim)
+                             (split-string)))
+    (when pos (push "[ ]" words))
+    (regexp-opt words)))
 
 (defun org-mem-parser--mk-id (file-attribs entry-string)
   "Return a bignum that represents an entry.
@@ -395,7 +403,15 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
                                       (nth 1 dir-or-cache)
                                     (dir-locals-read-from-dir
                                      (file-name-directory file)))))
-                 (all-locals (append (hack-local-variables--find-variables)
+                 ;; FIXME: `hack-local-variables--find-variables' can cause
+                 ;; errors if you have a #+BEGIN_SRC block containing a block
+                 ;; of Local Variables, especially if it is mis-formatted in
+                 ;; any way.
+                 ;; We do `ignore-errors' here just in case, but that'll also
+                 ;; skip any actual Local Variables at the end of file when
+                 ;; there is such an error.
+                 ;; Unfortunate, but not horrible.
+                 (all-locals (append (ignore-errors (hack-local-variables--find-variables))
                                      (hack-local-variables-prop-line)
                                      (cdr (assq 'org-mode dir-class-vars))
                                      (cdr (assq 'text-mode dir-class-vars))
