@@ -1,12 +1,13 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "org-mem" "0.29.1"
+(define-package "org-mem" "0.31.3"
   "Fast info from a large number of Org file contents."
-  '((emacs  "29.1")
-    (el-job "2.7.1")
-    (llama  "0.5.0"))
+  '((emacs          "29.1")
+    (el-job         "2.7.3")
+    (llama          "1.0")
+    (truename-cache "0.1.2"))
   :url "https://github.com/meedstrom/org-mem"
-  :commit "3f9c030396844c976bd5d80f08d0a3aa42962486"
-  :revdesc "0.29.1-0-g3f9c03039684"
+  :commit "444dc77d7f2b8ca42b2526b2d2b93fa7ff626df5"
+  :revdesc "0.31.3-0-g444dc77d7f2b"
   :keywords '("text")
   :authors '(("Martin Edström" . "meedstrom@runbox.eu"))
   :maintainers '(("Martin Edström" . "meedstrom@runbox.eu")))
