@@ -20,13 +20,6 @@
 ;; Optional mechanisms to update the tables in just-in-time fashion,
 ;; reducing our need to do `org-mem--full-scan' so often.
 
-;; Technically, repeating a full scan is never needed *IF* we use these hooks
-;; correctly.  However, that is hard and humans are fallible.
-
-;; If a full scan is sufficiently performant, you can just do it more often,
-;; instead of using these hooks at all.  It is also a simple way to detect
-;; filesystem changes made by other Emacsen or the command line.
-
 ;;; Code:
 
 (require 'cl-lib)
@@ -40,58 +33,77 @@
 
 ;;; Targeted-scan
 
-(defun org-mem-updater-update (&optional synchronous)
+(defun org-mem-updater-update (&optional synchronous files)
   "Update cache for each file that has changed, appeared or disappeared.
 
 If SYNCHRONOUS, block Emacs until done.
-If SYNCHRONOUS and interrupted by a quit, cancel the update."
-  (let* ((db-files (copy-hash-table org-mem--truename<>metadata))
-         (modified-files
-          (cl-loop for truename in (org-mem--list-files-from-fs)
-                   as attr = (file-attributes truename)
-                   as real-mtime = (and attr (file-attribute-modification-time attr))
+If interrupted by a quit while blocking, cancel the update.
+
+If FILES, scan only FILES specifically.
+This is a hack you should almost never need, but can be used to sidestep
+rare performance issues with scanning the filesystem.
+
+If you must pass FILES, be sure to include the true name of every file
+touched by what you were doing, such as refiling an Org subtree from one
+file into another, or renaming one file name to another.
+Do not refer to `buffer-file-truename' for this, because it is a lie.
+Be sure also to include the old true name of any files that have been
+deleted or renamed, so that they may be removed from org-mem tables.
+If you mess up the tables, use `org-mem-reset'."
+  (unless files
+    (setq files
+          (let* ((db-files (copy-hash-table org-mem--truename<>metadata))
+                 (modified-files
+                  (cl-loop
+                   for (truename . attr) in (org-mem--truenames-and-attrs)
+                   as real-mtime = (file-attribute-modification-time attr)
                    as db-mtime = (prog1 (org-mem-file-mtime truename)
                                    (remhash truename db-files))
                    when (or (not db-mtime)
                             (not (time-equal-p db-mtime real-mtime)))
                    collect truename))
-         (removed-files
-          (hash-table-keys db-files))
-         (all (append removed-files modified-files)))
-    ;; If async job already ongoing, this will cancel that and run a new one
+                 (removed-files
+                  (hash-table-keys db-files)))
+            (append removed-files modified-files))))
+  (when files
+    ;; If async job already ongoing, this will cancel that and run a new one.
+    ;; That's fine, because mtimes of those files being scanned won't have
+    ;; changed in our db, so we've automatically picked them up again.
     (el-job-ng-run
      :id 'org-mem-updater
      :inject-vars (append (org-mem--mk-work-vars)
                           (el-job-ng-vars org-mem-inject-vars))
      :require (cons 'org-mem-parser org-mem-load-features)
-     :inputs all
+     :inputs files
      :funcall-per-input #'org-mem-parser--parse-file
      :callback #'org-mem-updater--finalize-targeted-scan)
     (when synchronous
       (el-job-ng-await-or-die
        'org-mem-updater 3600
-       (format "Running org-mem-updater-update... (files: %S)" all)))))
+       (format "Running org-mem-updater-update... (files: %S)" files)))))
 
 (defun org-mem-updater--finalize-targeted-scan (parse-results)
   "Handle PARSE-RESULTS from `org-mem-updater-update'."
   (run-hook-with-args 'org-mem-pre-targeted-scan-functions parse-results)
   (mapc #'clrhash (hash-table-values org-mem--key<>subtable))
-  (let (bad-paths problems)
-    (with-current-buffer (setq org-mem-scratch (get-buffer-create " *org-mem-scratch*" t))
-      (cl-loop for (bad-path problem file-datum entries links) in parse-results do
-               (when bad-path (push bad-path bad-paths))
+  (let (problems)
+    (org-mem--invalidate-file-names
+     (cl-loop for (bad-path) in parse-results when bad-path collect bad-path))
+    (with-current-buffer (get-buffer-create " *org-mem-scratch*" t)
+      (cl-loop for (_ problem file-datum entries links) in parse-results do
                (when problem (push problem problems))
                (let ((file (car file-datum)))
-                 (dolist (entry (gethash file org-mem--truename<>entries))
-                   (remhash (org-mem-entry-id entry) org-mem--id<>entry)
-                   (remhash (org-mem-entry-title-maybe entry) org-mem--title<>id)
-                   (remhash (org-mem-entry-pseudo-id entry) org-mem--pseudo-id<>links)
-                   (run-hook-with-args 'org-mem--forget-entry-functions entry))
-                 (remhash file org-mem--truename<>entries)
-                 (remhash file org-mem--truename<>metadata)
-                 (run-hook-with-args 'org-mem--forget-file-functions file) ; Can deprecate
-                 (puthash file file-datum org-mem--truename<>metadata)
-                 (run-hook-with-args 'org-mem--record-file-functions file-datum))
+                 (when file
+                   (dolist (entry (gethash file org-mem--truename<>entries))
+                     (remhash (org-mem-entry-id entry) org-mem--id<>entry)
+                     (remhash (org-mem-entry-title-maybe entry) org-mem--title<>id)
+                     (remhash (org-mem-entry-pseudo-id entry) org-mem--pseudo-id<>links)
+                     (run-hook-with-args 'org-mem--forget-entry-functions entry))
+                   (remhash file org-mem--truename<>entries)
+                   (remhash file org-mem--truename<>metadata)
+                   (run-hook-with-args 'org-mem--forget-file-functions file) ; Can deprecate
+                   (puthash file file-datum org-mem--truename<>metadata)
+                   (run-hook-with-args 'org-mem--record-file-functions file-datum)))
                (dolist (entry entries)
                  (org-mem--record-entry entry)
                  (run-hook-with-args 'org-mem--record-entry-functions entry))
@@ -99,10 +111,6 @@ If SYNCHRONOUS and interrupted by a quit, cancel the update."
                  (push link (gethash (org-mem-link-entry-pseudo-id link)
                                      org-mem--pseudo-id<>links))
                  (run-hook-with-args 'org-mem--record-link-functions link))))
-    ;; REVIEW: Here we assume it is fine to invalidate now rather than before
-    ;; or during the above loop.  I'm not 100% sure it is.
-    ;; The reason to do it now: it's more efficient when called once on a list.
-    (org-mem--invalidate-file-names bad-paths)
     (org-mem--rebuild-specially-indexed-tables)
 
     (run-hook-with-args 'org-mem-post-targeted-scan-functions parse-results)
@@ -110,6 +118,61 @@ If SYNCHRONOUS and interrupted by a quit, cancel the update."
       (setq org-mem--problems (append problems org-mem--problems))
       (message "Scan had problems, see M-x org-mem-list-problems"))))
 
+
+;;; Mode
+
+(defvar org-mem-updater--reset-timer (timer-create)
+  "Timer for intermittently running `org-mem--scan-full'.")
+
+(defun org-mem-updater-adjust-reset-timer (&rest _)
+  "Adjust `org-mem-updater--reset-timer' based on duration of last full scan.
+If timer not running, start it.
+Override this if you prefer different timer delays, or no timer."
+  (let ((new-delay (* 10 (1+ org-mem--time-elapsed))))
+    (when (or (not (member org-mem-updater--reset-timer timer-idle-list))
+              ;; Don't enter an infinite loop -- idle timers can be a footgun.
+              (not (> (float-time (or (current-idle-time) 0))
+                      new-delay)))
+      (cancel-timer org-mem-updater--reset-timer)
+      (setq org-mem-updater--reset-timer
+            (run-with-idle-timer new-delay t #'org-mem--scan-full)))))
+
+(defvar org-mem-updater--debounce-timer nil)
+(defun org-mem-updater--update-soon (&optional file &rest _)
+  "Schedule to run `org-mem-updater-update' very soon.
+
+Designed for `after-save-hook' and as advice for `delete-file' and
+`rename-file'.  Such functions might be called many times in a loop,
+and this design is meant to avoid invoking `org-mem-updater-update'
+for every FILE, but wait and do a massed invocation afterwards."
+  (setq file (or file buffer-file-name))
+  (when (and file (cl-some (##string-suffix-p % file) org-mem-suffixes))
+    (if (memq org-mem-updater--debounce-timer timer-list)
+      (timer-set-time org-mem-updater--debounce-timer
+                      (time-add (current-time) 0.5))
+    (setq org-mem-updater--debounce-timer
+          (run-with-timer 0.5 nil #'org-mem-updater-update)))))
+
+;;;###autoload
+(define-minor-mode org-mem-updater-mode
+  "Keep Org-mem cache up to date."
+  :global t
+  :group 'org-mem
+  (require 'org-mem-updater)
+  (if org-mem-updater-mode
+      (progn
+        (add-hook 'org-mem-post-full-scan-functions #'org-mem-updater-adjust-reset-timer 90)
+        (add-hook 'after-save-hook                  #'org-mem-updater--update-soon)
+        (advice-add 'rename-file :after             #'org-mem-updater--update-soon)
+        (advice-add 'delete-file :after             #'org-mem-updater--update-soon)
+        (org-mem-updater-adjust-reset-timer)
+        (org-mem--scan-full))
+    (remove-hook 'org-mem-post-full-scan-functions #'org-mem-updater-adjust-reset-timer)
+    (remove-hook 'after-save-hook                  #'org-mem-updater--update-soon)
+    (advice-remove 'rename-file                    #'org-mem-updater--update-soon)
+    (advice-remove 'delete-file                    #'org-mem-updater--update-soon)
+    (cancel-timer org-mem-updater--reset-timer)))
+
 
 ;;; Instant placeholders
 
@@ -134,26 +197,11 @@ If SYNCHRONOUS and interrupted by a quit, cancel the update."
 (defvar org-trust-scanner-tags)
 (defvar org-use-tag-inheritance)
 
-(defun org-mem-updater-ensure-buffer-file-known ()
-  "Record basic file metadata if not already known.
-Use this if you cannot wait for `org-mem-updater-mode' to pick it up."
-  (require 'org)
-  (when (and buffer-file-name
-             (derived-mode-p 'org-mode)
-             (file-exists-p buffer-file-name))
-    (let ((file (file-truename buffer-file-name)))
-      (unless (gethash file org-mem--truename<>metadata)
-        (puthash file
-                 (list file
-                       (file-attributes file)
-                       (line-number-at-pos (point-max) t)
-                       (point-max))
-                 org-mem--truename<>metadata)))))
-
 (defun org-mem-updater-ensure-link-at-point-known (&rest _)
   "Record the link at point.
 Use this if you cannot wait for `org-mem-updater-mode' to pick it up.
 No support for citations."
+  (declare (obsolete nil "0.30.0 (2026-02-18)"))
   (require 'org)
   (require 'org-element-ast)
   (when (and buffer-file-name
@@ -174,6 +222,7 @@ No support for citations."
 (defun org-mem-updater-ensure-id-node-at-point-known ()
   "Record ID-node at point.
 Use this if you cannot wait for `org-mem-updater-mode' to pick it up."
+  (declare (obsolete nil "0.30.0 (2026-02-18)"))
   (require 'org)
   (require 'ol)
   (when (and buffer-file-name
@@ -190,11 +239,29 @@ Use this if you cannot wait for `org-mem-updater-mode' to pick it up."
             (org-mem-updater-ensure-buffer-file-known)
             (org-mem--record-entry (org-mem-updater-mk-entry-atpt))))))))
 
+(defun org-mem-updater-ensure-buffer-file-known ()
+  "Record basic file metadata if not already known.
+Use this if you cannot wait for `org-mem-updater-mode' to pick it up."
+  (declare (obsolete nil "0.30.0 (2026-02-18)"))
+  (require 'org)
+  (when (and buffer-file-name
+             (derived-mode-p 'org-mode)
+             (file-exists-p buffer-file-name))
+    (let ((file (file-truename buffer-file-name)))
+      (unless (gethash file org-mem--truename<>metadata)
+        (puthash file
+                 (list file
+                       (file-attributes file)
+                       (line-number-at-pos (point-max) t)
+                       (point-max))
+                 org-mem--truename<>metadata)))))
+
 (defun org-mem-updater-mk-link-atpt ()
   "Return an `org-mem-link' object appropriate for link at point.
 It is not associated with any entries or files, however.
 Return nil if no link at point.
 No support for citations."
+  (declare (obsolete nil "0.30.0 (2026-02-18)"))
   (require 'org)
   (require 'org-element-ast)
   (if-let* ((el (org-element-context))
@@ -224,6 +291,7 @@ No support for citations."
   "Return an `org-mem-entry' object appropriate for entry at point.
 It is not associated with any links or files, however.
 Some fields are incomplete or left at nil."
+  (declare (obsolete nil "0.30.0 (2026-02-18)"))
   (require 'org)
   (require 'ol)
   (let* ((heading (org-get-heading t t t t))
@@ -280,6 +348,7 @@ Some fields are incomplete or left at nil."
 
 (defun org-mem-updater--tags-at-point-inherited-only ()
   "Like `org-get-tags', but get only the inherited tags."
+  (declare (obsolete nil "0.30.0 (2026-02-18)"))
   (require 'org)
   (let ((all-tags (if org-use-tag-inheritance
                       (org-get-tags)
@@ -290,66 +359,6 @@ Some fields are incomplete or left at nil."
              when (get-text-property 0 'inherited tag)
              collect (substring-no-properties tag))))
 
-
-;;; Mode
-
-(defvar org-mem-updater--reset-timer (timer-create)
-  "Timer for intermittently running `org-mem--scan-full'.")
-
-(defun org-mem-updater-adjust-reset-timer (&rest _)
-  "Adjust `org-mem-updater--reset-timer' based on duration of last full scan.
-If timer not running, start it.
-Override this if you prefer different timer delays, or no timer."
-  (let ((new-delay (* 10 (1+ org-mem--time-elapsed))))
-    (when (or (not (member org-mem-updater--reset-timer timer-idle-list))
-              ;; Don't enter an infinite loop -- idle timers can be a footgun.
-              (not (> (float-time (or (current-idle-time) 0))
-                      new-delay)))
-      (cancel-timer org-mem-updater--reset-timer)
-      (setq org-mem-updater--reset-timer
-            (run-with-idle-timer new-delay t #'org-mem--scan-full)))))
-
-(defun org-mem-updater--update-file-soon (&optional file &rest _)
-  "Schedule to run `org-mem-updater-update' very soon.
-
-Designed for `after-save-hook' and as advice for `delete-file' and
-`rename-file'.  Such functions might be called many times in a loop,
-and this design is meant to avoid invoking `org-mem-updater-update'
-for every FILE, but wait and do a massed invocation afterwards."
-  (setq file (or file buffer-file-name))
-  (when (and file (cl-some (##string-suffix-p % file) org-mem-suffixes))
-    (org-mem-updater--update-soon)))
-
-(defvar org-mem-updater--debounce-timer nil)
-(defun org-mem-updater--update-soon ()
-  "Schedule to run `org-mem-updater-update' very soon.
-If already scheduled, postpone to very soon."
-  (if (memq org-mem-updater--debounce-timer timer-list)
-      (timer-set-time org-mem-updater--debounce-timer
-                      (time-add (current-time) 0.5))
-    (setq org-mem-updater--debounce-timer
-          (run-with-timer 0.5 nil #'org-mem-updater-update))))
-
-;;;###autoload
-(define-minor-mode org-mem-updater-mode
-  "Keep Org-mem cache up to date."
-  :global t
-  :group 'org-mem
-  (require 'org-mem-updater)
-  (if org-mem-updater-mode
-      (progn
-        (add-hook 'org-mem-post-full-scan-functions #'org-mem-updater-adjust-reset-timer 90)
-        (add-hook 'after-save-hook                  #'org-mem-updater--update-file-soon)
-        (advice-add 'rename-file :after             #'org-mem-updater--update-file-soon)
-        (advice-add 'delete-file :after             #'org-mem-updater--update-file-soon)
-        (org-mem-updater-adjust-reset-timer)
-        (org-mem--scan-full))
-    (remove-hook 'org-mem-post-full-scan-functions #'org-mem-updater-adjust-reset-timer)
-    (remove-hook 'after-save-hook                  #'org-mem-updater--update-file-soon)
-    (advice-remove 'rename-file                    #'org-mem-updater--update-file-soon)
-    (advice-remove 'delete-file                    #'org-mem-updater--update-file-soon)
-    (cancel-timer org-mem-updater--reset-timer)))
-
 (defvar org-mem-updater--id-or-ref-target<>old-links :obsolete)
 (defvar org-mem-updater--new-id-or-ref-targets :obsolete)
 (define-obsolete-function-alias 'org-mem-updater--adjust-timer #'org-mem-updater-adjust-reset-timer "0.28.0 (2026-02-07)")
